#!/usr/bin/perl

use strict;
use Test::More tests=>4;
use lib 'lib';
use_ok('Fry::Lib::CDBI::Load');
use_ok('Fry::Lib::CDBI::Basic');
use_ok('Fry::Lib::CDBI::Tags');
use_ok('Fry::Lib::CDBI::Outline');
